@@ -31,7 +31,6 @@
 #include "ephy-settings.h"
 #include "ephy-smaps.h"
 #include "ephy-snapshot-service.h"
-#include "ephy-vcs-version.h"
 #include "ephy-web-app-utils.h"
 
 #include <gio/gio.h>
@@ -163,10 +162,10 @@ ephy_about_handler_handle_about (EphyAboutHandler       *handler,
   char *version;
   GtkIconInfo *icon_info;
 
-  version = g_strdup_printf (_("Version %s"), VCSVERSION);
+  version = g_strdup_printf (_("Version %s"), VERSION);
 
   icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default (),
-                                          "org.gnome.Epiphany",
+                                          APPLICATION_ID,
                                           256,
                                           GTK_ICON_LOOKUP_FORCE_SVG);
 
@@ -175,7 +174,7 @@ ephy_about_handler_handle_about (EphyAboutHandler       *handler,
                           "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS "\" rel=\"stylesheet\" type=\"text/css\">"
                           "</head><body>"
                           "<div class=\"dialog\">"
-                          "<img src=\"file://%s\"/>"
+                          "<img id=\"about-icon\" src=\"file://%s\"/>"
                           "<h1 id=\"about-title\">%s</h1>"
                           "<h2 id=\"about-subtitle\">%s</h2>"
                           "<p id=\"about-tagline\">%s</p>"
@@ -192,7 +191,7 @@ ephy_about_handler_handle_about (EphyAboutHandler       *handler,
 #endif
                           version,
                           _("A simple, clean, beautiful view of the web"),
-                          "WebKitGTK+", webkit_get_major_version (), webkit_get_minor_version (), webkit_get_micro_version ());
+                          "WebKitGTK", webkit_get_major_version (), webkit_get_minor_version (), webkit_get_micro_version ());
   g_free (version);
   if (icon_info)
     g_object_unref (icon_info);
@@ -237,41 +236,68 @@ handle_applications_finished_cb (EphyAboutHandler       *handler,
   GList *applications, *p;
 
   data_str = g_string_new (NULL);
-  g_string_append_printf (data_str, "<html><head><title>%s</title>"
-                          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
-                          "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS "\" rel=\"stylesheet\" type=\"text/css\">"
-                          "<script>"
-                          "  function deleteWebApp(appID) {"
-                          "    window.webkit.messageHandlers.aboutApps.postMessage(appID);"
-                          "    var row = document.getElementById(appID);"
-                          "    row.parentNode.removeChild(row);"
-                          "  }"
-                          "</script>"
-                          "</head><body class=\"applications-body\"><h1>%s</h1>"
-                          "<p>%s</p>",
-                          _("Applications"),
-                          _("Applications"),
-                          _("List of installed web applications"));
-
-  g_string_append (data_str, "<table>");
-
   applications = g_task_propagate_pointer (G_TASK (result), NULL);
-  for (p = applications; p; p = p->next) {
-    EphyWebApplication *app = (EphyWebApplication *)p->data;
 
+  if (g_list_length (applications) > 0) {
+    g_string_append_printf (data_str, "<html><head><title>%s</title>"
+                            "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
+                            "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS "\" rel=\"stylesheet\" type=\"text/css\">"
+                            "<script>"
+                            "  function deleteWebApp(appID) {"
+                            "    window.webkit.messageHandlers.aboutApps.postMessage(appID);"
+                            "    var row = document.getElementById(appID);"
+                            "    row.parentNode.removeChild(row);"
+                            "  }"
+                            "</script>"
+                            "</head><body class=\"applications-body\"><h1>%s</h1>"
+                            "<p>%s</p>",
+                            _("Applications"),
+                            _("Applications"),
+                            _("List of installed web applications"));
+
+    g_string_append (data_str, "<table>");
+
+    for (p = applications; p; p = p->next) {
+      EphyWebApplication *app = (EphyWebApplication *)p->data;
+
+      g_string_append_printf (data_str,
+                              "<tbody><tr id =\"%s\">"
+                              "<td class=\"icon\"><img width=64 height=64 src=\"file://%s\"></img></td>"
+                              "<td class=\"data\"><div class=\"appname\">%s</div><div class=\"appurl\">%s</div></td>"
+                              "<td class=\"input\"><input type=\"button\" value=\"%s\" onclick=\"deleteWebApp('%s');\"></td>"
+                              "<td class=\"date\">%s <br /> %s</td></tr></tbody>",
+                              app->id, app->icon_url, app->name, app->url, _("Delete"), app->id,
+                              /* Note for translators: this refers to the installation date. */
+                              _("Installed on:"), app->install_date);
+    }
+
+    g_string_append (data_str, "</table></body></html>");
+  } else {
+    g_autoptr(GtkIconInfo) icon_info = NULL;
+    g_autofree gchar *icon = g_strconcat ("application-x-addon-symbolic", NULL);
+
+    g_string_append_printf (data_str, "<html><head><title>%s</title>"
+                            "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
+                            "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS "\" rel=\"stylesheet\" type=\"text/css\">"
+                            "</head><body class=\"applications-body\">",
+                            _("Applications"));
+
+    icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default (),
+                                            icon,
+                                            128,
+                                            0);
     g_string_append_printf (data_str,
-                            "<tbody><tr id =\"%s\">"
-                            "<td class=\"icon\"><img width=64 height=64 src=\"file://%s\"></img></td>"
-                            "<td class=\"data\"><div class=\"appname\">%s</div><div class=\"appurl\">%s</div></td>"
-                            "<td class=\"input\"><input type=\"button\" value=\"%s\" onclick=\"deleteWebApp('%s');\"></td>"
-                            "<td class=\"date\">%s <br /> %s</td></tr></tbody>",
-                            app->id, app->icon_url, app->name, app->url, _("Delete"), app->id,
-                            /* Note for translators: this refers to the installation date. */
-                            _("Installed on:"), app->install_date);
+                            "  <div id=\"overview\" class=\"overview-empty\">\n"
+                            "    <img src=\"file://%s\"/>\n"
+                            "    <div><h1>%s</h1></div>\n"
+                            "    <div><p>%s</p></div>\n"
+                            "  </div>\n"
+                            "</body></html>\n",
+                            icon_info ? gtk_icon_info_get_filename (icon_info) : "",
+                            /* Displayed when opening applications without any installed web apps. */
+                            _("Applications"), _("You can add your favorite website by clicking <b>Install Site as Web Application…</b> within page menu."));
   }
 
-  g_string_append (data_str, "</table></body></html>");
-
   ephy_web_application_free_application_list (applications);
 
   data_length = data_str->len;
@@ -343,9 +369,10 @@ history_service_query_urls_cb (EphyHistoryService     *history,
 
   if (g_list_length (urls) == 0 || !success) {
     GtkIconInfo *icon_info;
+    g_autofree gchar *icon = g_strconcat (APPLICATION_ID, "-symbolic", NULL);
 
     icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default (),
-                                            "org.gnome.Epiphany-symbolic",
+                                            icon,
                                             128,
                                             0);
     g_string_append_printf (data_str,
@@ -373,7 +400,7 @@ history_service_query_urls_cb (EphyHistoryService     *history,
 
     snapshot = ephy_snapshot_service_lookup_cached_snapshot_path (snapshot_service, url->url);
     if (snapshot)
-      thumbnail_style = g_strdup_printf (" style=\"background: url(file://%s) no-repeat;\"", snapshot);
+      thumbnail_style = g_strdup_printf (" style=\"background: url(file://%s) no-repeat; background-size: 100%%;\"", snapshot);
     else
       ephy_embed_shell_schedule_thumbnail_update (shell, url);
 
@@ -446,13 +473,11 @@ ephy_about_handler_handle_incognito (EphyAboutHandler       *handler,
                           "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS "\" rel=\"stylesheet\" type=\"text/css\">\n"
                           "</head>\n"
                           "<body class=\"incognito-body\">\n"
-                          "  <div id=\"mainblock\">\n"
-                          "    <div style=\"background: transparent url(ephy-resource:///org/gnome/epiphany/private-mode.svg) no-repeat 10px center;\">\n" \
-                          "      <h1>%s</h1>\n"
-                          "      <p>%s</p>\n"
-                          "      <p><strong>%s</strong> %s</p>\n"
-                          "    </div>\n"
-                          "  </div>\n"
+                          "  <img class=\"incognito-body-image\" src=\"ephy-resource:///org/gnome/epiphany/private-mode.svg\">\n" \
+                          "  <br/>\n"
+                          "  <h1>%s</h1>\n"
+                          "  <p>%s</p>\n"
+                          "  <p><strong>%s</strong> %s</p>\n"
                           "</body>\n"
                           "</div>\n"
                           "</html>\n",
