@@ -471,6 +471,11 @@ static const PrefData webkit_pref_entries[] =
     EPHY_PREFS_WEB_USER_AGENT,
     "user-agent",
     webkit_pref_callback_user_agent },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_MOBILE_USER_AGENT,
+    "mobile-user-agent",
+    /* No need for a specific callback, we just want to update the user agent. */
+    webkit_pref_callback_user_agent },
   { EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_COOKIES_POLICY,
     "accept-policy",
@@ -492,6 +497,7 @@ ephy_embed_prefs_init (gpointer user_data)
                                                        "enable-dns-prefetching", TRUE,
                                                        "enable-mediasource", TRUE,
                                                        "javascript-can-open-windows-automatically", TRUE,
+                                                       "enable-back-forward-navigation-gestures", TRUE,
                                                        NULL);
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
