@@ -304,7 +304,7 @@ create_desktop_file (const char *id,
   g_key_file_set_value (file, "Desktop Entry", "StartupNotify", "true");
   g_key_file_set_value (file, "Desktop Entry", "Terminal", "false");
   g_key_file_set_value (file, "Desktop Entry", "Type", "Application");
-  g_key_file_set_value (file, "Desktop Entry", "Categories", "Network;GNOME;GTK;");
+  g_key_file_set_value (file, "Desktop Entry", "Categories", "GNOME;GTK;");
 
   if (icon) {
     GOutputStream *stream;
@@ -627,7 +627,7 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
           if (g_file_test (app_file, G_FILE_TEST_EXISTS))
             applications = g_list_prepend (applications, app);
           else
-            g_free (app);
+            g_object_unref (app);
         } else
           applications = g_list_prepend (applications, app);
       }
