@@ -24,6 +24,8 @@
 #include "ephy-history-service-private.h"
 #include "ephy-history-types.h"
 #include "ephy-lib-type-builtins.h"
+#include "ephy-prefs.h"
+#include "ephy-settings.h"
 #include "ephy-sqlite-connection.h"
 #include "ephy-sync-utils.h"
 
@@ -325,7 +327,7 @@ ephy_history_service_message_new (EphyHistoryService           *service,
                                   EphyHistoryJobCallback        callback,
                                   gpointer                      user_data)
 {
-  EphyHistoryServiceMessage *message = g_slice_alloc0 (sizeof (EphyHistoryServiceMessage));
+  EphyHistoryServiceMessage *message = g_new0 (EphyHistoryServiceMessage, 1);
 
   message->service = service;
   message->type = type;
@@ -347,7 +349,7 @@ ephy_history_service_message_free (EphyHistoryServiceMessage *message)
   if (message->cancellable)
     g_object_unref (message->cancellable);
 
-  g_slice_free1 (sizeof (EphyHistoryServiceMessage), message);
+  g_free (message);
 }
 
 static void
@@ -523,7 +525,7 @@ signal_emission_context_free (SignalEmissionContext *ctx)
   g_object_unref (ctx->service);
   if (ctx->destroy_func && ctx->user_data)
     ctx->destroy_func (ctx->user_data);
-  g_slice_free (SignalEmissionContext, ctx);
+  g_free (ctx);
 }
 
 static SignalEmissionContext *
@@ -531,7 +533,7 @@ signal_emission_context_new (EphyHistoryService *service,
                              gpointer            user_data,
                              GDestroyNotify      destroy_func)
 {
-  SignalEmissionContext *ctx = g_slice_new0 (SignalEmissionContext);
+  SignalEmissionContext *ctx = g_new0 (SignalEmissionContext, 1);
 
   ctx->service = g_object_ref (service);
   ctx->user_data = user_data;
@@ -877,7 +879,7 @@ ephy_history_service_execute_set_url_zoom_level (EphyHistoryService *self,
 void
 ephy_history_service_set_url_zoom_level (EphyHistoryService    *self,
                                          const char            *url,
-                                         const double           zoom_level,
+                                         double                 zoom_level,
                                          GCancellable          *cancellable,
                                          EphyHistoryJobCallback callback,
                                          gpointer               user_data)
@@ -888,6 +890,10 @@ ephy_history_service_set_url_zoom_level (EphyHistoryService    *self,
   g_assert (EPHY_IS_HISTORY_SERVICE (self));
   g_assert (url != NULL);
 
+  /* Ensure that a change value which equals default zoom level is stored as 0.0 */
+  if (zoom_level == g_settings_get_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL))
+    zoom_level = 0.0f;
+
   variant = g_variant_new ("(sd)", url, zoom_level);
 
   message = ephy_history_service_message_new (self, SET_URL_ZOOM_LEVEL,
