@@ -69,6 +69,7 @@ struct _EphyLocationEntry {
   gboolean reader_mode_active;
 
   char *saved_text;
+  char *jump_tab;
 
   guint hash;
 
@@ -297,6 +298,7 @@ ephy_location_entry_finalize (GObject *object)
   EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (object);
 
   g_free (entry->saved_text);
+  g_clear_pointer (&entry->jump_tab, g_free);
 
   G_OBJECT_CLASS (ephy_location_entry_parent_class)->finalize (object);
 }
@@ -501,35 +503,44 @@ entry_key_press_cb (GtkEntry          *entry,
   if (event->keyval == GDK_KEY_Return ||
       event->keyval == GDK_KEY_KP_Enter ||
       event->keyval == GDK_KEY_ISO_Enter) {
-    g_autofree gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (location_entry->url_entry)));
-    gchar *url = g_strstrip (text);
-    g_autofree gchar *new_url = NULL;
-
-    if (strlen (url) > 5 && g_str_has_prefix (url, "http:") && url[5] != '/')
-      new_url = g_strdup_printf ("http://%s", url + 5);
-    else if (strlen (url) > 6 && g_str_has_prefix (url, "https:") && url[6] != '/')
-      new_url = g_strdup_printf ("https://%s", url + 6);
-
-    if (new_url) {
+    if (location_entry->jump_tab) {
       g_signal_handlers_block_by_func (location_entry->url_entry, G_CALLBACK (editable_changed_cb), location_entry);
-      gtk_entry_set_text (GTK_ENTRY (location_entry->url_entry), new_url);
+      gtk_entry_set_text (GTK_ENTRY (location_entry->url_entry), location_entry->jump_tab);
       g_signal_handlers_unblock_by_func (location_entry->url_entry, G_CALLBACK (editable_changed_cb), location_entry);
-    }
-
-    if (state == GDK_CONTROL_MASK) {
+      g_clear_pointer (&location_entry->jump_tab, g_free);
+    } else {
       g_autofree gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (location_entry->url_entry)));
       gchar *url = g_strstrip (text);
+      g_autofree gchar *new_url = NULL;
 
-      /* Remove control mask to prevent opening address in a new window */
-      event->state &= ~GDK_CONTROL_MASK;
+      gtk_entry_set_text (GTK_ENTRY (entry), location_entry->jump_tab ? location_entry->jump_tab : text);
 
-      if (!g_utf8_strchr (url, -1, ' ') && !g_utf8_strchr (url, -1, '.')) {
-        g_autofree gchar *new_url = g_strdup_printf ("www.%s.com", url);
+      if (strlen (url) > 5 && g_str_has_prefix (url, "http:") && url[5] != '/')
+        new_url = g_strdup_printf ("http://%s", url + 5);
+      else if (strlen (url) > 6 && g_str_has_prefix (url, "https:") && url[6] != '/')
+        new_url = g_strdup_printf ("https://%s", url + 6);
 
+      if (new_url) {
         g_signal_handlers_block_by_func (location_entry->url_entry, G_CALLBACK (editable_changed_cb), location_entry);
         gtk_entry_set_text (GTK_ENTRY (location_entry->url_entry), new_url);
         g_signal_handlers_unblock_by_func (location_entry->url_entry, G_CALLBACK (editable_changed_cb), location_entry);
       }
+
+      if (state == GDK_CONTROL_MASK) {
+        g_autofree gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (location_entry->url_entry)));
+        gchar *url = g_strstrip (text);
+
+        /* Remove control mask to prevent opening address in a new window */
+        event->state &= ~GDK_CONTROL_MASK;
+
+        if (!g_utf8_strchr (url, -1, ' ') && !g_utf8_strchr (url, -1, '.')) {
+          g_autofree gchar *new_url = g_strdup_printf ("www.%s.com", url);
+
+          g_signal_handlers_block_by_func (location_entry->url_entry, G_CALLBACK (editable_changed_cb), location_entry);
+          gtk_entry_set_text (GTK_ENTRY (location_entry->url_entry), new_url);
+          g_signal_handlers_unblock_by_func (location_entry->url_entry, G_CALLBACK (editable_changed_cb), location_entry);
+        }
+      }
     }
 
     ephy_location_entry_activate (location_entry);
@@ -725,7 +736,7 @@ button_box_size_allocated_cb (GtkWidget    *widget,
   GtkStateFlags state_flags;
   GtkBorder padding;
 
-  if (lentry->allocation_width == allocation->width)
+  if (lentry->allocation_width == (guint)allocation->width)
     return;
 
   lentry->allocation_width = allocation->width;
@@ -773,8 +784,11 @@ ephy_location_entry_suggestion_activated (DzlSuggestionEntry *entry,
 {
   EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (user_data);
   DzlSuggestion *suggestion = dzl_suggestion_entry_get_suggestion (entry);
+  const gchar *text = ephy_suggestion_get_uri (EPHY_SUGGESTION (suggestion));
+
   g_signal_handlers_block_by_func (entry, G_CALLBACK (editable_changed_cb), user_data);
-  gtk_entry_set_text (GTK_ENTRY (entry), ephy_suggestion_get_uri (EPHY_SUGGESTION (suggestion)));
+  gtk_entry_set_text (GTK_ENTRY (entry), lentry->jump_tab ? lentry->jump_tab : text);
+  g_clear_pointer (&lentry->jump_tab, g_free);
   g_signal_handlers_unblock_by_func (entry, G_CALLBACK (editable_changed_cb), user_data);
 
   g_signal_stop_emission_by_name (entry, "suggestion-activated");
@@ -794,7 +808,14 @@ suggestion_selected (DzlSuggestionEntry *entry,
   const gchar *uri = dzl_suggestion_get_id (suggestion);
 
   g_signal_handlers_block_by_func (entry, G_CALLBACK (editable_changed_cb), user_data);
-  gtk_entry_set_text (GTK_ENTRY (entry), uri);
+  g_clear_pointer (&lentry->jump_tab, g_free);
+
+  if (g_str_has_prefix (uri, "ephy-tab://")) {
+    lentry->jump_tab = g_strdup (uri);
+    gtk_entry_set_text (GTK_ENTRY (entry), dzl_suggestion_get_subtitle (suggestion));
+  } else {
+    gtk_entry_set_text (GTK_ENTRY (entry), uri);
+  }
   gtk_editable_set_position (GTK_EDITABLE (entry), -1);
   g_signal_handlers_unblock_by_func (entry, G_CALLBACK (editable_changed_cb), user_data);
 
@@ -828,6 +849,7 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 
   /* URL entry */
   entry->url_entry = dzl_suggestion_entry_new ();
+  gtk_entry_set_icon_tooltip_text (GTK_ENTRY (entry->url_entry), GTK_ENTRY_ICON_PRIMARY, _("Show website security status and permissions"));
 
   /* Add special widget css provider */
   context = gtk_widget_get_style_context (GTK_WIDGET (entry->url_entry));
@@ -867,6 +889,7 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   /* Bookmark */
   entry->bookmark_event_box = gtk_event_box_new ();
   entry->bookmark = gtk_image_new_from_icon_name ("non-starred-symbolic", GTK_ICON_SIZE_MENU);
+  gtk_widget_set_tooltip_text (entry->bookmark, _("Bookmark this page"));
   gtk_widget_show (entry->bookmark);
   g_signal_connect (G_OBJECT (entry->bookmark_event_box), "button_press_event", G_CALLBACK (bookmark_icon_button_press_event_cb), entry);
   gtk_container_add (GTK_CONTAINER(entry->bookmark_event_box), entry->bookmark);
@@ -878,6 +901,7 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   /* Reader Mode */
   entry->reader_mode_event_box = gtk_event_box_new ();
   entry->reader_mode = gtk_image_new_from_icon_name ("ephy-reader-mode-symbolic", GTK_ICON_SIZE_MENU);
+  gtk_widget_set_tooltip_text (entry->reader_mode, _("Toggle reader mode"));
   gtk_widget_show (entry->reader_mode);
   gtk_container_add (GTK_CONTAINER(entry->reader_mode_event_box), entry->reader_mode);
   gtk_box_pack_end (GTK_BOX (button_box), entry->reader_mode_event_box, FALSE, FALSE, 6);
