@@ -257,7 +257,8 @@ ephy_notebook_switch_page_cb (GtkNotebook *notebook,
   GtkWidget *child;
 
   child = gtk_notebook_get_nth_page (notebook, page_num);
-  gtk_widget_grab_focus (child);
+  if (!ephy_web_view_is_in_auth_dialog (ephy_embed_get_web_view (EPHY_EMBED (child))))
+    gtk_widget_grab_focus (child);
 
   /* Remove the old page, we dont want to grow unnecessarily
    * the list */
@@ -466,17 +467,16 @@ show_tabs_changed_cb (GSettings    *settings,
   update_tabs_visibility (nb, FALSE);
 }
 
-static guint
-get_last_pinned_tab_pos (GtkNotebook *notebook)
+static int
+get_last_pinned_tab_pos (EphyNotebook *notebook)
 {
-  gint pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
-  gint i;
-  gint found = -1;
+  int pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+  int found = -1;
 
-  for (i = 0; i < pages; i++) {
+  for (int i = 0; i < pages; i++) {
     GtkWidget *child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i);
 
-    if (ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), EPHY_EMBED (child)))
+    if (ephy_notebook_tab_is_pinned (notebook, EPHY_EMBED (child)))
       found = i;
   }
 
@@ -489,11 +489,15 @@ page_reordered_cb (GtkNotebook *notebook,
                    guint        page_num,
                    gpointer     user_data)
 {
-  guint last_pinned_tab_pos = get_last_pinned_tab_pos (notebook);
+  int last_pinned_tab_pos = get_last_pinned_tab_pos (EPHY_NOTEBOOK (notebook));
+  gboolean is_current_tab_pinned = ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), EPHY_EMBED (child));
 
   /* Ensure that pinned tabs will always stay at the beginning of tab bar */
-  if (last_pinned_tab_pos != -1 && page_num <= last_pinned_tab_pos)
-    gtk_notebook_reorder_child (notebook, child, last_pinned_tab_pos);
+  if (last_pinned_tab_pos != -1 && page_num <= (guint)last_pinned_tab_pos) {
+    gint new_pos = is_current_tab_pinned ? last_pinned_tab_pos : last_pinned_tab_pos + 1;
+
+    gtk_notebook_reorder_child (notebook, child, new_pos);
+  }
 }
 
 static void
@@ -516,7 +520,7 @@ ephy_notebook_init (EphyNotebook *notebook)
                           G_CALLBACK (ephy_notebook_switch_page_cb),
                           NULL);
   g_signal_connect (notebook, "page-reordered",
-                    (GCallback)page_reordered_cb, NULL);
+                    G_CALLBACK (page_reordered_cb), NULL);
 
   /* Set up drag-and-drop target */
   g_signal_connect (notebook, "drag-data-received",
@@ -967,20 +971,19 @@ ephy_notebook_tab_set_pinned (EphyNotebook *notebook,
   GtkWidget *tab_label;
   gboolean expanded;
 
+  gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (notebook), embed, !is_pinned);
+  tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (notebook), embed);
+  ephy_tab_label_set_pinned (tab_label, is_pinned);
+
   if (is_pinned) {
     gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), embed, 0);
     expanded = FALSE;
   } else {
     expanded = g_settings_get_boolean (EPHY_SETTINGS_UI, EPHY_PREFS_UI_EXPAND_TABS_BAR);
-    page_reordered_cb (GTK_NOTEBOOK (notebook), embed, 0, NULL);
+    gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), embed, -1);
   }
 
   gtk_container_child_set (GTK_CONTAINER (notebook), embed, "tab-expand", expanded, NULL);
-
-  gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (notebook), embed, !is_pinned);
-
-  tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (notebook), embed);
-  ephy_tab_label_set_pinned (tab_label, is_pinned);
 }
 
 gboolean
