@@ -57,8 +57,9 @@ static int do_step_n = -1;
 static int migration_version = -1;
 static char *profile_dir = NULL;
 
-/* The legacy dir is used by everything before version 29 which migrates
- * to the new directory */
+/* The legacy dir is used by everything before version 30, which migrates
+ * to the new directory.
+ */
 static const char *
 legacy_default_profile_dir (void)
 {
@@ -74,11 +75,16 @@ legacy_profile_dir (void)
   static char *dir = NULL;
   if (dir == NULL)
     {
-      /* If this isn't actually a legacy dir it starts at a later migrating step anyway */
+      /* FIXME: This is fragile and not correct.
+       */
       if (profile_dir != NULL)
-        dir = profile_dir;
+        {
+          dir = profile_dir;
+        }
       else
-        dir = (char *)legacy_default_profile_dir ();
+        {
+          dir = (char *)legacy_default_profile_dir ();
+        }
     }
   return dir;
 }
@@ -1016,7 +1022,7 @@ migrate_zoom_level (void)
   char *history_filename;
   const char *sql_query;
 
-  history_filename = g_build_filename (ephy_profile_dir (), EPHY_HISTORY_FILE, NULL);
+  history_filename = g_build_filename (legacy_profile_dir (), EPHY_HISTORY_FILE, NULL);
   if (!g_file_test (history_filename, G_FILE_TEST_EXISTS))
     goto out;
 
@@ -1083,6 +1089,17 @@ out:
     g_error_free (error);
 }
 
+static void
+migrate_web_extension_config_dir (void)
+{
+  /* Epiphany 3.32.0 web process created its config dir in the wrong place by
+   * mistake for default profile dirs. It only contains read-only data copied
+   * from dconf, so we can just delete it.
+   */
+  g_autofree char *path = g_build_filename (ephy_default_profile_dir (), "config", NULL);
+  ephy_file_delete_dir_recursively (path, NULL);
+}
+
 static gboolean
 move_directory_contents (const char *source_path,
                          const char *dest_path)
@@ -1131,117 +1148,15 @@ move_directory_contents (const char *source_path,
   return TRUE;
 }
 
-static void
-migrate_profile_directories (void)
-{
-  GList *web_apps, *l;
-
-  /* Web app profiles moved from config to data dirs. If this migration has
-   * already been run for a separate profile, then the legacy application list
-   * should be empty, so it's harmless to run this multiple times.
-   */
-  web_apps = ephy_web_application_get_legacy_application_list ();
-  for (l = web_apps; l; l = l->next) {
-    EphyWebApplication *app = (EphyWebApplication *)l->data;
-    g_autoptr(GError) error = NULL;
-
-    g_autofree char *old_name = g_strconcat ("app-epiphany-", app->id, NULL);
-    g_autofree char *old_path = g_build_filename (legacy_profile_dir (), old_name, NULL);
-    g_autofree char *app_path = ephy_web_application_get_profile_directory (app->id);
-
-    if (!move_directory_contents (old_path, app_path))
-      continue;
-
-    // Create an empty file to indicate it's an app
-    g_autofree char *app_file = g_build_filename (app_path, ".app", NULL);
-    int fd = g_open (app_file, O_WRONLY|O_CREAT|O_TRUNC, 0644);
-    if (fd < 0)
-      g_warning ("Failed to create .app file: %s", g_strerror (errno));
-    else
-      close (fd);
-
-    // Update Exec and Icon to point to the new profile dir
-    g_autofree char *old_profile_prefix = g_build_filename (legacy_profile_dir (), "app-epiphany-", NULL);
-    g_autofree char *new_profile_prefix = g_build_filename (g_get_user_data_dir (), "epiphany-", NULL);
-    g_autoptr(GKeyFile) file = g_key_file_new ();
-    g_autofree char *desktop_file_path = g_build_filename (app_path, app->desktop_file, NULL);
-    g_key_file_load_from_file (file, desktop_file_path, G_KEY_FILE_NONE, NULL);
-
-    g_autofree char *exec = g_key_file_get_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_EXEC, &error);
-    if (exec == NULL) {
-      g_warning ("Failed to get Exec key from %s: %s", desktop_file_path, error->message);
-      continue;
-    }
-    g_autofree char *new_exec = ephy_string_find_and_replace (exec, old_profile_prefix, new_profile_prefix);
-    LOG ("migrate_profile_directories: setting Exec to %s", new_exec);
-    g_key_file_set_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_EXEC, new_exec);
-
-    g_autofree char *icon = g_key_file_get_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, &error);
-    if (exec == NULL) {
-      g_warning ("Failed to get Icon key from %s: %s", desktop_file_path, error->message);
-      continue;
-    }
-    g_autofree char *new_icon = ephy_string_find_and_replace (icon, old_profile_prefix, new_profile_prefix);
-    LOG ("migrate_profile_directories: setting Icon to %s", new_icon);
-    g_key_file_set_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, new_icon);
-
-    if (!g_key_file_save_to_file (file, desktop_file_path, &error))
-      g_warning ("Failed to save desktop file %s", error->message);
-
-    g_autofree char *desktop_symlink_path = g_build_filename (g_get_user_data_dir (),
-                                                              "applications",
-                                                              app->desktop_file,
-                                                              NULL);
-    g_autoptr(GFile) desktop_symlink = g_file_new_for_path (desktop_symlink_path);
-    LOG ("Symlinking %s to %s", desktop_symlink_path, desktop_file_path);
-
-    // Try removing old symlink, failure is ok assuming it doesn't exist.
-    if (!g_file_delete (desktop_symlink, NULL, &error)) {
-      g_warning ("Failed to remove old symbolic link: %s", error->message);
-      g_clear_error (&error);
-    }
-
-    if (!g_file_make_symbolic_link (desktop_symlink, desktop_file_path, NULL, &error))
-      g_warning ("Failed to make symbolic link: %s", error->message);
-  }
-
-  ephy_web_application_free_application_list (web_apps);
-
-  /* The default profile also changed directories. This needs to run only once,
-   * for the main browser instance.
-   */
-  if (!ephy_profile_dir_is_default ())
-    return;
-
-  if (!move_directory_contents (legacy_default_profile_dir (), ephy_default_profile_dir ()))
-    return;
-
-  /* We are also moving some cache directories so just remove the old ones */
-  g_autoptr(GFile) new_directory = g_file_new_for_path (ephy_default_profile_dir ());
-  g_autoptr(GFile) adblock_directory = g_file_get_child (new_directory, "adblock");
-  g_file_delete (adblock_directory, NULL, NULL);
-  g_autoptr(GFile) gsb_file = g_file_get_child (new_directory, "gsb-threats.db");
-  g_file_delete (gsb_file, NULL, NULL);
-  g_autoptr(GFile) gsb_journal_file = g_file_get_child (new_directory, "gsb-threats.db-journal");
-  g_file_delete (gsb_journal_file, NULL, NULL);
-}
-
-static void
-migrate_web_extension_config_dir (void)
-{
-  /* Epiphany 3.32.0 web process created its config dir in the wrong place by
-   * mistake for default profile dirs. It only contains read-only data copied
-   * from dconf, so we can just delete it.
-   */
-  g_autofree char *path = g_build_filename (ephy_default_profile_dir (), "config", NULL);
-  ephy_file_delete_dir_recursively (path, NULL);
-}
-
 static void
 migrate_webapps_harder (void)
 {
   /* We created some webapps in the default profile directory by mistake.
    * Whoopsie!
+   *
+   * This migrator used to come after migrate_profile_directories(), which
+   * contained this bug. But now migrate_profile_directories() has to be run
+   * later to correct a different bug in earlier versions.
    */
   g_autoptr(GFileEnumerator) children = NULL;
   g_autoptr(GFileInfo) info;
@@ -1320,6 +1235,101 @@ migrate_webapps_harder (void)
   }
 }
 
+static void
+migrate_profile_directories (void)
+{
+  GList *web_apps, *l;
+
+  /* Web app profiles moved from config to data dirs. If this migration has
+   * already been run for a separate profile, then the legacy application list
+   * should be empty, so it's harmless to run this multiple times.
+   */
+  web_apps = ephy_web_application_get_legacy_application_list ();
+  for (l = web_apps; l; l = l->next) {
+    EphyWebApplication *app = (EphyWebApplication *)l->data;
+    g_autoptr(GError) error = NULL;
+
+    g_autofree char *old_name = g_strconcat ("app-epiphany-", app->id, NULL);
+    g_autofree char *old_path = g_build_filename (legacy_default_profile_dir (), old_name, NULL);
+    g_autofree char *app_path = ephy_web_application_get_profile_directory (app->id);
+
+    if (!move_directory_contents (old_path, app_path))
+      continue;
+
+    // Create an empty file to indicate it's an app
+    g_autofree char *app_file = g_build_filename (app_path, ".app", NULL);
+    int fd = g_open (app_file, O_WRONLY|O_CREAT|O_TRUNC, 0644);
+    if (fd < 0)
+      g_warning ("Failed to create .app file: %s", g_strerror (errno));
+    else
+      close (fd);
+
+    // Update Exec and Icon to point to the new profile dir
+    g_autofree char *old_profile_prefix = g_build_filename (legacy_default_profile_dir (), "app-epiphany-", NULL);
+    g_autofree char *new_profile_prefix = g_build_filename (g_get_user_data_dir (), "epiphany-", NULL);
+    g_autoptr(GKeyFile) file = g_key_file_new ();
+    g_autofree char *desktop_file_path = g_build_filename (app_path, app->desktop_file, NULL);
+    g_key_file_load_from_file (file, desktop_file_path, G_KEY_FILE_NONE, NULL);
+
+    g_autofree char *exec = g_key_file_get_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_EXEC, &error);
+    if (exec == NULL) {
+      g_warning ("Failed to get Exec key from %s: %s", desktop_file_path, error->message);
+      continue;
+    }
+    g_autofree char *new_exec = ephy_string_find_and_replace (exec, old_profile_prefix, new_profile_prefix);
+    LOG ("migrate_profile_directories: setting Exec to %s", new_exec);
+    g_key_file_set_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_EXEC, new_exec);
+
+    g_autofree char *icon = g_key_file_get_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, &error);
+    if (exec == NULL) {
+      g_warning ("Failed to get Icon key from %s: %s", desktop_file_path, error->message);
+      continue;
+    }
+    g_autofree char *new_icon = ephy_string_find_and_replace (icon, old_profile_prefix, new_profile_prefix);
+    LOG ("migrate_profile_directories: setting Icon to %s", new_icon);
+    g_key_file_set_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, new_icon);
+
+    if (!g_key_file_save_to_file (file, desktop_file_path, &error))
+      g_warning ("Failed to save desktop file %s", error->message);
+
+    g_autofree char *desktop_symlink_path = g_build_filename (g_get_user_data_dir (),
+                                                              "applications",
+                                                              app->desktop_file,
+                                                              NULL);
+    g_autoptr(GFile) desktop_symlink = g_file_new_for_path (desktop_symlink_path);
+    LOG ("Symlinking %s to %s", desktop_symlink_path, desktop_file_path);
+
+    // Try removing old symlink, failure is ok assuming it doesn't exist.
+    if (!g_file_delete (desktop_symlink, NULL, &error)) {
+      g_warning ("Failed to remove old symbolic link: %s", error->message);
+      g_clear_error (&error);
+    }
+
+    if (!g_file_make_symbolic_link (desktop_symlink, desktop_file_path, NULL, &error))
+      g_warning ("Failed to make symbolic link: %s", error->message);
+  }
+
+  ephy_web_application_free_application_list (web_apps);
+
+  /* The default profile also changed directories. This needs to run only once,
+   * for the main browser instance.
+   */
+  if (!ephy_profile_dir_is_default ())
+    return;
+
+  if (!move_directory_contents (legacy_default_profile_dir (), ephy_default_profile_dir ()))
+    return;
+
+  /* We are also moving some cache directories so just remove the old ones */
+  g_autoptr(GFile) new_directory = g_file_new_for_path (ephy_default_profile_dir ());
+  g_autoptr(GFile) adblock_directory = g_file_get_child (new_directory, "adblock");
+  g_file_delete (adblock_directory, NULL, NULL);
+  g_autoptr(GFile) gsb_file = g_file_get_child (new_directory, "gsb-threats.db");
+  g_file_delete (gsb_file, NULL, NULL);
+  g_autoptr(GFile) gsb_journal_file = g_file_get_child (new_directory, "gsb-threats.db-journal");
+  g_file_delete (gsb_journal_file, NULL, NULL);
+}
+
 static void
 migrate_nothing (void)
 {
@@ -1413,6 +1423,15 @@ ephy_migrator (void)
     m ();
   }
 
+  if (!g_file_test (ephy_profile_dir (), G_FILE_TEST_EXISTS)) {
+    LOG ("Original profile directory does not exist. This is an expected"
+         " failure. Probably a web app is being migrated before the default"
+         " profile, and its profile directory was moved during the migration."
+         " Epiphany must be restarted with the new profile directory. The"
+         " migration will be run again.");
+    return FALSE;
+  }
+
   if (ephy_profile_utils_set_migration_version (EPHY_PROFILE_MIGRATION_VERSION) != TRUE) {
     LOG ("Failed to store the current migration version");
     return FALSE;
